# -*- coding: iso-8859-15 -*-

# Contractor, a contract validation tool
#    Copyright (C) 2009  Guido de Caso, Alexis Tcach, Víctor Braberman, Diego Garbervetsky, Sebastián Uchitel
#    {gdecaso, atcach, vbraber, diegog, suchitel}@dc.uba.ar
#    LaFHIS - Departamento de Computación - Facultad de Ciencias Exactas y Naturales - Universidad de Buenos Aires
#    Pabellón I - Ciudad Universitaria - (C1428EGA) - Buenos Aires - Argentina
#    Tel: +54-11-4576-3390 eext. 704
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from reader.options_parser import Options

def ltsa_write_fsm(fsm, contract):
	
	# write header
	print "// autogenerated with contractor (http://lafhis.dc.uba.ar/contractor)"
	print ""
	
	# write contract name and link to initial state (if it exists)
	print contract.name + " =",
	if fsm.initial_state == None:
		print "STOP."
	else:
		print fsm.initial_state + ","
		
		# write states
		states_size = len(fsm.states)
		for s in range(states_size):
			print fsm.states[s].name + " =", 
			
			# print state remarks
			state_remarks = ""
			if fsm.states[s].uncertain:
				state_remarks += "may be an inconsistent state"
			if Options.options.ltsa_print_invariants:
				if state_remarks != "":
					state_remarks +=", "
				state_remarks += "invariant: " + fsm.states[s].invariant
			if state_remarks != "":
				print "/* " + state_remarks + " */"
				print "\t",
			
			# write transitions for this state
			transitions_size = len(fsm.states[s].transitions)
			if transitions_size == 0:
				print "STOP",
			else:
				print "(",
				for t in range(transitions_size):
					if t > 0:
						print "\t|",
					print fsm.states[s].transitions[t].action + " -> " + fsm.states[s].transitions[t].destination,
					
					# print transition remarks
					transition_remarks = ""
					if fsm.states[s].transitions[t].uncertain:
						transition_remarks += "transition may not exist"
					if fsm.states[s].transitions[t].violates_invariant:
						if transition_remarks != "":
							transition_remarks +=", "
						transition_remarks += "may violate contract invariant on destination"
					if transition_remarks != "":
						print "\t/* " + transition_remarks + " */",

					if t < transitions_size - 1:
						print ""
				print ")",
		
			# comma or dot?
			if s < states_size - 1:
				print ","
			else:
				print "."
